/// <reference types="node" />
import { Network } from "./Network";
/**
 * Address encoding/decoding utility for legacy and segwit addresses.
 */
export declare class Address {
    /**
     * Encodes a P2PKH or P2SH address using Base58Check in the format
     * - 1-byte: prefix
     * - 20-byte: hash160 of data
     * - 4-byte: checksum from hash256 of data
     * @param prefix - 1-byte prefix from a Network config (either p2)
     * @param hash
     * @returns
     */
    static encodeBase58(prefix: number, hash: Buffer): string;
    /**
     * Decodes a Base58Check encoded legacy address and determines the
     * network. This function returns the hash160 and the prefix.
     * @param encoded base58check encoded string
     * @returns
     */
    static decodeBase58(encoded: string): {
        network: Network;
        prefix: number;
        hash: Buffer;
    };
    /**
     * Encodes a native segwit P2WPKH or P2WSH address using bech32.
     * @param prefix hrp part of the address
     * @param version segwit version
     * @param program witness program
     * @returns
     */
    static encodeBech32(prefix: string, version: number, program: Buffer): string;
    /**
     * Decodes a bech32 address and returns the corresponding network,
     * segwit version, and witness program.
     * @param encoded
     * @returns
     */
    static decodeBech32(encoded: string): {
        network: Network;
        version: number;
        program: Buffer;
    };
}
