"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.BitcoinError = void 0;
const BitcoinErrorCode_1 = require("./BitcoinErrorCode");
function getMessage(code) {
    switch (code) {
        case BitcoinErrorCode_1.BitcoinErrorCode.InvalidPrivateKey:
            return "Invalid private key";
        case BitcoinErrorCode_1.BitcoinErrorCode.Base58ChecksumFailed:
            return "Base58Check checksum failed";
        case BitcoinErrorCode_1.BitcoinErrorCode.PubKeyInvalid:
            return "Invalid public key";
        case BitcoinErrorCode_1.BitcoinErrorCode.PubKeyHashInvalid:
            return "Invalid pubkeyhash";
        case BitcoinErrorCode_1.BitcoinErrorCode.SigEncodingInvalid:
            return "Signatures requires BIP66 DER encoding";
        case BitcoinErrorCode_1.BitcoinErrorCode.SigHashTypeInvalid:
            return "Invalid Signature SIGHASH type";
        case BitcoinErrorCode_1.BitcoinErrorCode.MultiSigSetupInvalid:
            return "MultiSig structure is invalid";
        case BitcoinErrorCode_1.BitcoinErrorCode.Hash160Invalid:
            return "Hash160 requires 20-byte Buffer";
        case BitcoinErrorCode_1.BitcoinErrorCode.Hash256Invalid:
            return "Hash256 requires 32-byte Buffer";
        case BitcoinErrorCode_1.BitcoinErrorCode.NetworkMismatch:
            return "Network mismatch";
        case BitcoinErrorCode_1.BitcoinErrorCode.InputIndexOutOfRange:
            return "Input index out of range";
        case BitcoinErrorCode_1.BitcoinErrorCode.UnkownHdKeyVersion:
            return "Unkown HD key version";
        case BitcoinErrorCode_1.BitcoinErrorCode.InvalidHdEncoding:
            return "Invalid HD encoding";
        case BitcoinErrorCode_1.BitcoinErrorCode.InvalidHdPath:
            return "Invalid HD key path";
        case BitcoinErrorCode_1.BitcoinErrorCode.InvalidHdDerivation:
            return "Invalid HD key derivation";
        case BitcoinErrorCode_1.BitcoinErrorCode.InvalidHdPrivateKey:
            return "Invalid HD private key";
        case BitcoinErrorCode_1.BitcoinErrorCode.InvalidHdPublicKey:
            return "Invalid HD public key";
        case BitcoinErrorCode_1.BitcoinErrorCode.InvalidBech32Encoding:
            return "Invalid bech32 encoding";
        case BitcoinErrorCode_1.BitcoinErrorCode.InvalidBech32Hrp:
            return "Invalid bech32 human readable part";
        case BitcoinErrorCode_1.BitcoinErrorCode.InvalidBech32Checksum:
            return "Invalid bech32 checksum";
        case BitcoinErrorCode_1.BitcoinErrorCode.UnknownAddressPrefix:
            return "Unknown address prefix";
        case BitcoinErrorCode_1.BitcoinErrorCode.InvalidSegwitVersion:
            return "Unknown segwit version";
        case BitcoinErrorCode_1.BitcoinErrorCode.InvalidWitnessProgram:
            return "Invalid witness program";
        case BitcoinErrorCode_1.BitcoinErrorCode.InvalidMnemonicWordList:
            return "Invalid mnemonic word list";
        case BitcoinErrorCode_1.BitcoinErrorCode.InvalidMnemonicEntropy:
            return "Invalid mnemonic entropy";
        case BitcoinErrorCode_1.BitcoinErrorCode.InvalidMnemonicWord:
            return "Unknown mnemonic word";
        case BitcoinErrorCode_1.BitcoinErrorCode.InvalidMnemonicChecksum:
            return "Mnemonic checksum failed";
        case BitcoinErrorCode_1.BitcoinErrorCode.InvalidWifEncoding:
            return "Invalid WIF encoding";
        case BitcoinErrorCode_1.BitcoinErrorCode.UnknownWifPrefix:
            return "Unknown WIF prefix";
        case BitcoinErrorCode_1.BitcoinErrorCode.ValueUnderflow:
            return "Value underflow";
    }
}
class BitcoinError extends Error {
    constructor(code, info) {
        super(getMessage(code));
        this.code = code;
        this.info = info;
        this.name = "BitcoinError";
    }
}
exports.BitcoinError = BitcoinError;
//# sourceMappingURL=BitcoinError.js.map