/// <reference types="node" />
import { Bits } from "./Bits";
import { BlockHeader } from "./BlockHeader";
import { HashValue } from "./HashValue";
import { Tx } from "./Tx";
/**
 * This class represents a Bitcoin block including fully constructed
 * transactions. It can be used to deserialize a raw transaction receive
 * from a peer or deserialized from HEX.
 *
 * This class extends the `BlockHeader` type since all properties on
 * a `BlockHeader` are part of a `Block`.
 */
export declare class Block extends BlockHeader {
    /**
     * Parses a Block with full transactions information by first
     * parsing the `BlockHeader` then the transactions.
     * @param buf
     */
    static fromBuffer(buf: Buffer): Block;
    /**
     * Complete list of transactions, in the order they weere included
     * in the `Block`.
     */
    txs: Tx[];
    constructor(version: number, previousBlockHash: HashValue, merkleRoot: HashValue, timestamp: number, bits: Bits, nonce: number, txs: Tx[]);
    /**
     * Serializes the `Block` into a Buffer by first serializing the
     * `BlockHeader` followed by a `varint` that encodes the number of
     * transactions followed by a sequence of `Tx`.
     */
    toBuffer(): Buffer;
}
