import { HdKeyType } from "./HdKeyType";
import { Network } from "./Network";
import { HdPrivateKey } from "./HdPrivateKey";
import { HdPublicKey } from "./HdPublicKey";
export declare class HdKeyCodec {
    static decodeVersion(version: number): [Network, HdKeyType, boolean];
    /**
     * Decodes an extended private key or public key from the
     * serialization format defined in BIP32.
     *
     * For example:
     * xprv9s21ZrQH143K3QTDL4LXw2F7HEK3wJUD2nW2nRk4stbPy6cq3jPPqjiChkVvvNKmPGJxWUtg6LnF5kejMRNNU3TGtRBeJgk33yuGBxrMPHi
     * xpub661MyMwAqRbcFtXgS5sYJABqqG9YLmC4Q1Rdap9gSE8NqtwybGhePY2gZ29ESFjqJoCu1Rupje8YtGqsefD265TMg7usUDFdp6W1EGMcet8
     *
     * @remarks
     *
     * The value uses a prefix of xprv, yprv, or zprv and is Base58Check.
     *
     * ```
     * The format includes:
     * [4 byte]: version
     * [1 byte]: depth
     * [4 byte]: parent fingerprint
     * [4 byte]: number
     * [32 byte]: chaincode
     * [33 byte]: key
     * ```
     *
     * The 33-byte key values uses a 0x00 prefix plus the 32-byte private
     * key.
     *
     * @param input encoded input
     *
     * @throws {@link BitcoinError} throws when there is an invalid
     * encoding, bad checksum, or if you attempt to decode a public key.
     *
     * @returns an instance of the extended private key or public key
     */
    static decode(input: string): HdPrivateKey | HdPublicKey;
    /**
     * Encodes either a {@link HdPrivateKey} or {@link HdPublicKey}
     * according to BIP32.
     *
     * For example:
     * xprv9s21ZrQH143K3QTDL4LXw2F7HEK3wJUD2nW2nRk4stbPy6cq3jPPqjiChkVvvNKmPGJxWUtg6LnF5kejMRNNU3TGtRBeJgk33yuGBxrMPHi
     * xpub661MyMwAqRbcFtXgS5sYJABqqG9YLmC4Q1Rdap9gSE8NqtwybGhePY2gZ29ESFjqJoCu1Rupje8YtGqsefD265TMg7usUDFdp6W1EGMcet8
     *
     * @remarks
     * The value uses a prefix of xprv, yprv, or zprv and is Base58Check
     * encoded.
     *
     * The format includes:
     * ```
     * [4 byte]: version
     * [1 byte]: depth
     * [4 byte]: parent fingerprint
     * [4 byte]: number
     * [32 byte]: chaincode
     * [33 byte]: key
     * ```
     *
     * For private keys, the 33-byte key value is prefixed with 0x00.
     *
     * @returns Base58Check encoded extended key
     */
    static encode(key: HdPrivateKey | HdPublicKey): string;
}
