"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.HdPublicKey = void 0;
const crypto = __importStar(require("@node-lightning/crypto"));
const BitcoinError_1 = require("./BitcoinError");
const BitcoinErrorCode_1 = require("./BitcoinErrorCode");
const bufio_1 = require("@node-lightning/bufio");
const HdKeyCodec_1 = require("./HdKeyCodec");
const HdKeyType_1 = require("./HdKeyType");
const HARDENED_INDEX = 0x80000000;
/**
 * A hierarchical deterministic extended public key as defined in BIP32.
 * This class allows derivation of unhardened public keys. It may be
 * generated from a derivation, decoded, or obtained a parent.
 * {@link HdPrivateKey}.
 */
class HdPublicKey {
    /**
     * Decodes an extended private key from the serialization format
     * defined in BIP32.
     *
     * For example:
     * xpub661MyMwAqRbcFtXgS5sYJABqqG9YLmC4Q1Rdap9gSE8NqtwybGhePY2gZ29ESFjqJoCu1Rupje8YtGqsefD265TMg7usUDFdp6W1EGMcet8
     *
     * @remarks
     *
     * The value uses a prefix of xprv, yprv, or zprv and is Base58Check.
     *
     * ```
     * The format includes:
     * [4 byte]: version
     * [1 byte]: depth
     * [4 byte]: parent fingerprint
     * [4 byte]: number
     * [32 byte]: chaincode
     * [33 byte]: key
     * ```
     *
     * @param input encoded input
     *
     * @throws {@link BitcoinError} throws when there is an invalid
     * encoding, bad checksum, or if you attempt to decode a private key.
     *
     * @returns an instance of the extended public key
     */
    static decode(input) {
        const result = HdKeyCodec_1.HdKeyCodec.decode(input);
        if (!(result instanceof HdPublicKey)) {
            throw new BitcoinError_1.BitcoinError(BitcoinErrorCode_1.BitcoinErrorCode.InvalidHdPublicKey, input);
        }
        return result;
    }
    /**
     * Gets the numeric version from the underlying network
     */
    get version() {
        switch (this.type) {
            case HdKeyType_1.HdKeyType.x:
                return this.network.xpubVersion;
            case HdKeyType_1.HdKeyType.y:
                return this.network.ypubVersion;
            case HdKeyType_1.HdKeyType.z:
                return this.network.zpubVersion;
        }
    }
    /**
     * Gets the network this public key is associated with
     */
    get network() {
        return this.publicKey.network;
    }
    /**
     * Returns true if the public key is hardened, meaning the number
     * is between 2^31 and 2^32-1.
     */
    get isHardened() {
        return this.number >= HARDENED_INDEX;
    }
    /**
     * Lazy loads the fingerprint of the public key. This is defined as
     * the `hash160(compressed_pubkey)`;
     */
    get fingerprint() {
        if (!this._fingerprint) {
            this._fingerprint = crypto.hash160(this.publicKey.toBuffer());
        }
        return this._fingerprint;
    }
    /**
     * This function computes a child extended public key from the parent
     * extened public key. It is only defined for non-hardened child keys.
     *
     * @remarks
     *
     * The derivation is defined in BIP32 using HMAC-SHA512
     * {@link https://github.com/bitcoin/bips/blob/master/bip-0032.mediawiki}
     *
     * The derivation uses the parent's chaincode as the key.  Then uses
     * `parent public key || i` as the HMAC data.
     *
     * The resulting 64-bytes are split where where the left half is
     * used to tweakAdd against the parent public key. The right half
     * is the child's chain_code.
     *
     * @param i key number to derive, must be less than 2^31-1
     *
     * @throws {@link BitcoinError} throws an `InvalidHdDerivation` error
     * code if there is an attempt to derive a hardened child key. Throws
     * if the tweak value is invalid.
     *
     *
     * @returns child non-hardened {@link HdPublicKey}
     */
    derive(i) {
        // From here on we're working with a public key, so we cannot
        // derive hardened public keys since they require the private
        // key as part of the derivation data.
        if (i >= HARDENED_INDEX) {
            throw new BitcoinError_1.BitcoinError(BitcoinErrorCode_1.BitcoinErrorCode.InvalidHdDerivation);
        }
        const data = new bufio_1.BufferWriter(Buffer.alloc(37));
        data.writeBytes(this.publicKey.toBuffer());
        data.writeUInt32BE(i);
        const l = crypto.hmac(this.chainCode, data.toBuffer(), "sha512");
        const ll = l.slice(0, 32);
        const lr = l.slice(32);
        const childPubKey = this.publicKey.tweakAdd(ll);
        const childChainCode = lr;
        const child = new HdPublicKey();
        child.type = this.type;
        child.depth = this.depth + 1;
        child.number = this.number;
        child.publicKey = childPubKey;
        child.chainCode = childChainCode;
        return child;
    }
    /**
     * Encodes the {@link HdPublicKey} according to BIP32.
     *
     * For example:
     * xpub661MyMwAqRbcFtXgS5sYJABqqG9YLmC4Q1Rdap9gSE8NqtwybGhePY2gZ29ESFjqJoCu1Rupje8YtGqsefD265TMg7usUDFdp6W1EGMcet8
     *
     * @remarks
     * The value uses a prefix of xprv, yprv, or zprv and is Base58Check
     * encoded.
     *
     * The format includes:
     * ```
     * [4 byte]: version
     * [1 byte]: depth
     * [4 byte]: parent fingerprint
     * [4 byte]: number
     * [32 byte]: chaincode
     * [33 byte]: key
     * ```
     * @returns Base56Check encoded extended public key
     */
    encode() {
        return HdKeyCodec_1.HdKeyCodec.encode(this);
    }
    /**
     * Returns the address encoded according to the type of HD key.
     * For x-type this returns a base58 encoded P2PKH address.
     * For y-type this returns a base58 encoded P2SH-P2WPKH address.
     * For z-type this returns a bech32 encoded P2WPKH address.
     * @returns encoded address
     */
    toAddress() {
        switch (this.type) {
            case HdKeyType_1.HdKeyType.x:
                return this.publicKey.toP2pkhAddress();
            case HdKeyType_1.HdKeyType.y:
                return this.publicKey.toP2nwpkhAddress();
            case HdKeyType_1.HdKeyType.z:
                return this.publicKey.toP2wpkhAddress();
        }
    }
    /**
     * Sugar for `instance.publicKey.toBuffer()` and returns the SEC
     * encoded public key in compressed or uncompressed format.
     * @returns
     */
    toSecBuffer() {
        return this.publicKey.toBuffer();
    }
    /**
     * Sugar for `instance.publicKey.toHex()` and returns the SEC
     * encoded public key as a hex string in compressed or uncompressed
     * format.
     * @returns
     */
    toSecHex() {
        return this.publicKey.toHex();
    }
}
exports.HdPublicKey = HdPublicKey;
//# sourceMappingURL=HdPublicKey.js.map